//
//  ContentView.swift
//  SCRepro
//
//  Created by Tyler Thompson on 8/1/22.
//

import SwiftUI
import SwiftCurrent
import SwiftCurrent_SwiftUI

struct ContentView: View {
    @StateObject var model = Model()
    var body: some View {
        WorkflowView(launchingWith: model) {
            WorkflowItem(FirstView.self)
                .presentationType(.navigationLink)
            WorkflowItem(SecondView.self)
                .presentationType(.navigationLink)
            WorkflowItem(ThirdView.self)
            WorkflowItem(FourthView.self)
        }
        .embedInNavigationView()
    }
}

struct FirstView: View, FlowRepresentable {
    typealias WorkflowOutput = ObservedObject<Model>
    weak var _workflowPointer: AnyFlowRepresentable?

    @ObservedObject var model: Model
    init(with model: Model) {
        _model = ObservedObject(wrappedValue: model)
    }

    var body: some View {
        VStack {
            Toggle("Test", isOn: $model.someBool)
            Button("Next") { proceedInWorkflow(_model) }
        }
    }
}

struct SecondView: View, FlowRepresentable {
    typealias WorkflowOutput = ObservedObject<Model>
    weak var _workflowPointer: AnyFlowRepresentable?

    @ObservedObject var model: Model
    init(with model: ObservedObject<Model>) {
        _model = model
    }

    func shouldLoad() -> Bool { model.someBool }

    var body: some View {
        VStack {
            Text(String(describing: Self.self))
            Button("Next") { proceedInWorkflow(_model) }
        }
    }
}

struct ThirdView: View, FlowRepresentable {
    typealias WorkflowOutput = ObservedObject<Model>
    weak var _workflowPointer: AnyFlowRepresentable?

    @ObservedObject var model: Model
    init(with model: ObservedObject<Model>) {
        _model = model
    }

    func shouldLoad() -> Bool { !model.someBool }

    var body: some View {
        VStack {
            Text(String(describing: Self.self))
            Button("Next") { proceedInWorkflow(_model) }
        }
    }
}

struct FourthView: View, FlowRepresentable {
    weak var _workflowPointer: AnyFlowRepresentable?

    var body: some View {
        VStack {
            Text(String(describing: Self.self))
        }
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
